@@ -1,32 +1,9 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 // Author: kenton@google.com (Kenton Varda)
 //  Based on original Protocol Buffers design by
@@ -38,6 +15,7 @@
 #include <string>
 
 #include "google/protobuf/compiler/java/field.h"
+#include "google/protobuf/io/printer.h"
 
 namespace google {
 namespace protobuf {
@@ -102,11 +80,12 @@ class ImmutableMessageFieldGenerator : public ImmutableFieldGenerator {
   virtual void PrintNestedBuilderCondition(
       io::Printer* printer, const char* regular_case,
       const char* nested_builder_case) const;
-  virtual void PrintNestedBuilderFunction(io::Printer* printer,
-                                          const char* method_prototype,
-                                          const char* regular_case,
-                                          const char* nested_builder_case,
-                                          const char* trailing_code) const;
+  virtual void PrintNestedBuilderFunction(
+      io::Printer* printer, const char* method_prototype,
+      const char* regular_case, const char* nested_builder_case,
+      const char* trailing_code,
+      absl::optional<io::AnnotationCollector::Semantic> semantic =
+          absl::nullopt) const;
 
  private:
   void GenerateKotlinOrNull(io::Printer* printer) const;
@@ -171,11 +150,12 @@ class RepeatedImmutableMessageFieldGenerator
   void PrintNestedBuilderCondition(
       io::Printer* printer, const char* regular_case,
       const char* nested_builder_case) const override;
-  void PrintNestedBuilderFunction(io::Printer* printer,
-                                  const char* method_prototype,
-                                  const char* regular_case,
-                                  const char* nested_builder_case,
-                                  const char* trailing_code) const override;
+  void PrintNestedBuilderFunction(
+      io::Printer* printer, const char* method_prototype,
+      const char* regular_case, const char* nested_builder_case,
+      const char* trailing_code,
+      absl::optional<io::AnnotationCollector::Semantic> semantic =
+          absl::nullopt) const override;
 };
 
 }  // namespace java
